<?php

/*******************************************************************************
Add the given IP to the .htaccess file and the banned IP DB table. This is a 
modified, stripped-down, version of the admin function by a similar name. The 
name was changed to prevent having to check if it was present, for speed.
/******************************************************************************/
function AlterHtaccessFile($ip) {
    if (! filter_var($ip, FILTER_VALIDATE_IP, ['flags' => FILTER_FLAG_IPV4]) ) {
        return false;
    }
    $arrayWrite = array();
    $htaccessfile = '';
    $arrayWrite = GetHtaccessFileContents($htaccessfile);       
    
    if ($arrayWrite) {    
        $endLine = '';
        $items = count($arrayWrite);

        for ($i = 0; $i < $items; ++$i) {
     
            if (strpos($arrayWrite[$i], $ip) !== FALSE) { //IP exists in .htaccess file - should never happen since the IP accessed the site now
                return false;
            }
            
            if (strpos(addslashes($arrayWrite[$i]), '# End IP deny by View Counter from www.oscommerce-solution.com') !== FALSE) {
                $endLine = $i;
                break;
            }                        
        }
        
        reset($arrayWrite);
        
        if ($endLine) {  //htaccess already has View Counter changes so add to them
            $arrayTmp = array();
            $arrayTmp[] = 'deny from ' . $ip . PHP_EOL;
            array_splice($arrayWrite, $endLine - 1, 0, $arrayTmp);
        } else {        
            $arrayWrite[] = '# Begin IP deny by View Counter from www.oscommerce-solution.com' . PHP_EOL;      
            $arrayWrite[] = 'order allow,deny' . PHP_EOL;
            $arrayWrite[] = 'deny from ' . $ip . PHP_EOL;
            $arrayWrite[] = 'allow from all' . PHP_EOL;
            $arrayWrite[] = '# End IP deny by View Counter from www.oscommerce-solution.com' . PHP_EOL;
        }
    } else { //create a new file
        $arrayWrite[] = '# Begin IP deny by View Counter from www.oscommerce-solution.com' . PHP_EOL;      
        $arrayWrite[] = 'order allow,deny' . PHP_EOL;
        $arrayWrite[] = 'deny from ' . $ip . PHP_EOL;
        $arrayWrite[] = 'allow from all' . PHP_EOL;
        $arrayWrite[] = '# End IP deny by View Counter from www.oscommerce-solution.com' . PHP_EOL;
    }

    if (! empty($arrayWrite)) {
        if (WriteHTAccess($htaccessfile, $arrayWrite)) {
            return true;
        } else {
            return false; //ERROR_HTACCESS_WRITE_FAILED;
        }
    }  
    return false;    
}

/*******************************************************************************
Return an array with city, state and country for the given IP to be used on the

/*******************************************************************************
Return the requested item based on the IP to be used on the
create account and new address pages.
/******************************************************************************/
function GetAutoFillArray($item = '') {
    require_once(DIR_FS_CATALOG . 'view_counter/IP2Location.php'); //PRE-PHOENIX ONLY
    $db = new \IP2Location\Database(DIR_FS_CATALOG . 'view_counter/' . VIEW_COUNTER_DATABASE, \IP2Location\Database::FILE_IO);
    //$records = $db->lookup($_SERVER['REMOTE_ADDR'], \IP2Location\Database::ALL);
    $records = $db->lookup('97.70.16.143', \IP2Location\Database::ALL);
    $addrArray = [];
    $countryID = '';
    
    if ($item == 'country') {
        $countries_query = tep_db_query("select countries_id from countries where countries_iso_code_2 = '" . tep_db_input($records['countryCode']) . "'");
        if (tep_db_num_rows($countries_query)) {
            $country = tep_db_fetch_array($countries_query);
            $countryID = $country['countries_id'];
        }  
    }
    
    $addrArray['city'] = (strpos($records['cityName'], 'This is a') === FALSE ? $records['cityName'] : '');
    $addrArray['state'] = (strpos($records['regionName'], 'This is a') === FALSE ? $records['regionName'] : '');
    $addrArray['zipcode'] = (strpos($records['zipCode'], 'This is a') === FALSE ? $records['zipCode'] : '');
    $addrArray['country'] = $countryID;
    
    return (isset($addrArray[$item]) ? $addrArray[$item] : null);
   // return ($addrArray[$item] ?? null);
 }
 
/*******************************************************************************
Load the .htaccess file into an array
/******************************************************************************/
function GetHtaccessFileContents(&$htaccessfile = '', $name_only = false) {
    $separator = ((substr(DIR_FS_CATALOG, -1) != '/') ? '/' : '');
    $htaccessfile = DIR_FS_CATALOG . $separator . '.htaccess';

    if ($name_only) {  //the contents are not needed so return
        return true;
    }

    if (file_exists($htaccessfile)) {
        $htaccessArray = file($htaccessfile);
        return $htaccessArray;
    }
    return false;
}

/******************************************************
A common function to handle invalid attempts. 
$hacker = false: the spoof message is sent.
$hacker = true:  the hacker message is sent.
******************************************************/
function HandleVCBan($ip, $hacker = false) { 
    
    $was_banned = (VIEW_COUNTER_BAD_BOT_TRAP != 'Both' ? TEXT_VIEW_COUNTER_EMAIL_NO : TEXT_VIEW_COUNTER_EMAIL_YES );
    $msg = ($hacker ? TEXT_VIEW_COUNTER_EMAIL_TRAP_MSG_HACKER : TEXT_VIEW_COUNTER_EMAIL_TRAP_MSG_SPOOF);
    $view_check_query = tep_db_query("select 1 from view_counter_banned where ip_number = INET_ATON('" . $ip . "')");
 
    if (tep_db_num_rows($view_check_query) == 0) { //hasn't been banned yet 

       if (VIEW_COUNTER_BAD_BOT_TRAP != 'Email') {  //then ban or both is set
            $sqlData = array('ip_number' => ip2long(tep_db_input($ip)), 'ignore_status' => 0);
            tep_db_perform('view_counter_banned', $sqlData);  
            AlterHtaccessFile($ip);
        } 
        
        if (VIEW_COUNTER_BAD_BOT_TRAP != 'Ban') {    //then send email. Ban, if set to Both, will be handled above
            $email_sent_query = tep_db_query("select 1 from view_counter_emails where ip_number = inet_aton('" . $ip . "') and sent = '1'");
            if (tep_db_num_rows($email_sent_query) == 0) { //shop owner hasn't been notified of this IP yet so do so
                $subject = sprintf(TEXT_VIEW_COUNTER_EMAIL_TRAP_SUBJECT, $ip); 
                $msg = sprintf($msg, $ip, 'http://www.projecthoneypot.org/ip_' . $ip, $was_banned);
                tep_db_query("insert into view_counter_emails (ip_number, sent) values (INET_ATON('" . $ip . "'), '1')");

                $vcSendTo = (tep_not_null(VIEW_COUNTER_SEND_EMAILS_TO) ? VIEW_COUNTER_SEND_EMAILS_TO : STORE_OWNER_EMAIL_ADDRESS);
                $accts = explode(',', $vcSendTo);
                $accts = array_map('trim', $accts);
                foreach ($accts as $sendto) {
                    tep_mail(STORE_OWNER, $sendto, $subject, $msg, STORE_OWNER, $vcSendTo);
                }
            }
        } 
    } else if (VIEW_COUNTER_BAD_BOT_TRAP != 'Ban') {    //has been banned but was the shop owner notified?
        $email_sent_query = tep_db_query("select 1 from view_counter_emails where ip_number = inet_aton('" . $ip . "') and sent = '1'");
        if (tep_db_num_rows($email_sent_query) == 0) { //shop owner hasn't been notified of this IP yet so do so
            $subject = sprintf(TEXT_VIEW_COUNTER_EMAIL_TRAP_SUBJECT, $ip); 
            $msg = sprintf($msg, $ip, 'http://www.projecthoneypot.org/ip_' .  $ip, $was_banned);
            tep_db_query("insert into view_counter_emails (ip_number, sent) values (INET_ATON('" . $ip . "'), '1')");

            $vcSendTo = (tep_not_null(VIEW_COUNTER_SEND_EMAILS_TO) ? VIEW_COUNTER_SEND_EMAILS_TO : STORE_OWNER_EMAIL_ADDRESS);
            $accts = explode(',', $vcSendTo);
            $accts = array_map('trim', $accts);
            foreach ($accts as $sendto) {
                tep_mail(STORE_OWNER, $sendto, $subject, $msg, STORE_OWNER, $vcSendTo);
            }    
        }     
    }      
} 

function IgnoreThisIP($ip) {
    $view_check_query = tep_db_query("select 1 from view_counter_ignore where ip_number = INET_ATON('" . tep_db_input($ip) . "')");
    if (tep_db_num_rows($view_check_query)) { //this IP is OK so ignore
        return true;
    }
    
    return ((strpos(VIEW_COUNTER_GOOD_IP_LIST, $ip) !== FALSE) ? true : false);
}
 
/*******************************************************************************
Write the altered .htaccess file
/******************************************************************************/
function WriteHTAccess($htaccessfile, $arrayWrite) {
    if (file_exists($htaccessfile)) {
        $fp = fopen($htaccessfile, "w" );
        if ($fp) {
            foreach ($arrayWrite as $line) {
                fputs($fp, $line);      
            }
            
            fclose($fp);
            return true;
        }
    }
    return false;
}